/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.       *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.portletcontainer.plugins.pc.aop;

import java.util.Iterator;

import javax.portlet.PortletContext;

import org.exoplatform.services.portletcontainer.plugins.pc.Constants;
import org.exoplatform.services.portletcontainer.plugins.pc.filter.PortletFilterChainImpl;
import org.exoplatform.services.portletcontainer.plugins.pc.filter.PortletFilterConfigImpl;
import org.exoplatform.services.portletcontainer.plugins.pc.filter.PortletFilterWrapper;
import org.exoplatform.services.portletcontainer.plugins.pc.portletAPIImp.ActionRequestImp;
import org.exoplatform.services.portletcontainer.plugins.pc.portletAPIImp.ActionResponseImp;
import org.exoplatform.services.portletcontainer.plugins.pc.portletAPIImp.EventRequestImp;
import org.exoplatform.services.portletcontainer.plugins.pc.portletAPIImp.EventResponseImp;
import org.exoplatform.services.portletcontainer.plugins.pc.portletAPIImp.RenderRequestImp;
import org.exoplatform.services.portletcontainer.plugins.pc.portletAPIImp.RenderResponseImp;
import org.exoplatform.services.portletcontainer.plugins.pc.portletAPIImp.ResourceRequestImp;
import org.exoplatform.services.portletcontainer.plugins.pc.portletAPIImp.ResourceResponseImp;

/*
 * @author: Benjamin Mestrallet
 * @author: Tuan Nguyen
 * @author: Roman Pedchenko
 */

public class PortletFilterCommand extends BaseCommandUnit {

  protected Object render(RenderExecutionContext rcontext)  throws Throwable {
    log_.debug("--> render method, call portlet filter aspect");
    
    RenderRequestImp req = (RenderRequestImp) rcontext.request_;
    RenderResponseImp res = (RenderResponseImp) rcontext.response_;
    PortletContext portletContext = req.getPortletConfig().getPortletContext();
    PortletFilterChainImpl chain = (PortletFilterChainImpl) req.getPortletDatas().getFilterChain();
    for (Iterator iterator = chain.getFiltersIterator(); iterator.hasNext();) {
      PortletFilterWrapper portletFilter = (PortletFilterWrapper) iterator.next();
      portletFilter.init(new PortletFilterConfigImpl(portletFilter.getFilterName(), portletFilter.getInitParam(), portletContext));
    }
    chain.restart();
    chain.doFilter(req, res);
    return rcontext.executeNextUnit();
  }

  protected Object processAction(ActionExecutionContext acontext)  throws Throwable {
    log_.debug("--> processAction method, call portlet filter aspect");
    
    ActionRequestImp req = (ActionRequestImp) acontext.request_;
    ActionResponseImp res = (ActionResponseImp) acontext.response_;
    PortletContext portletContext = req.getPortletConfig().getPortletContext();
    PortletFilterChainImpl chain = (PortletFilterChainImpl) req.getPortletDatas().getFilterChain();
    for (Iterator iterator = chain.getFiltersIterator(); iterator.hasNext();) {
      PortletFilterWrapper portletFilter = (PortletFilterWrapper) iterator.next();
      portletFilter.init(new PortletFilterConfigImpl(portletFilter.getFilterName(), portletFilter.getInitParam(), portletContext));
    }
    chain.restart();
    chain.doFilter(req, res);
    return acontext.executeNextUnit();
  }

  protected Object serveResource(ResourceExecutionContext rcontext)  throws Throwable {
    log_.debug("--> serveResource method, call portlet filter aspect");
    ResourceRequestImp req = (ResourceRequestImp) rcontext.request_;
    ResourceResponseImp res = (ResourceResponseImp) rcontext.response_;
    PortletContext portletContext = req.getPortletConfig().getPortletContext();
    PortletFilterChainImpl chain = (PortletFilterChainImpl) req.getPortletDatas().getFilterChain();
    for (Iterator iterator = chain.getFiltersIterator(); iterator.hasNext();) {
      PortletFilterWrapper portletFilter = (PortletFilterWrapper) iterator.next();
      portletFilter.init(new PortletFilterConfigImpl(portletFilter.getFilterName(), portletFilter.getInitParam(), portletContext));
    }
    chain.restart();
    chain.doFilter(req, res);
    return rcontext.executeNextUnit();
  }

  protected Object processEvent(EventExecutionContext econtext)  throws Throwable {
    log_.debug("--> processEvent method, call portlet filter aspect");
    
    EventRequestImp req = (EventRequestImp) econtext.request_;
    EventResponseImp res = (EventResponseImp) econtext.response_;
    PortletContext portletContext = req.getPortletConfig().getPortletContext();
    PortletFilterChainImpl chain = (PortletFilterChainImpl) req.getPortletDatas().getFilterChain();
    for (Iterator iterator = chain.getFiltersIterator(); iterator.hasNext();) {
      PortletFilterWrapper portletFilter = (PortletFilterWrapper) iterator.next();
      portletFilter.init(new PortletFilterConfigImpl(portletFilter.getFilterName(), portletFilter.getInitParam(), portletContext));
    }
    chain.restart();
    chain.doFilter(req, res);
    return econtext.executeNextUnit();
  }

}
