/**
 * Copyright 2001-2003 The eXo platform SARL All rights reserved.
 * Please look at license.txt in info directory for more license detail.
 **/

package org.exoplatform.services.portletcontainer.plugins.pc.filter;

import java.io.IOException;
import java.util.Iterator;
import java.util.List;
import javax.portlet.*;
import javax.portlet.filter.*;

import org.exoplatform.services.portletcontainer.pci.model.Filter;

/**
 * Created by The eXo Platform SARL
 * Author : Roman Pedchenko
 *          roman.pedchenko@exoplatform.com.ua
 */

public class PortletFilterWrapper implements ActionFilter, EventFilter, RenderFilter, ResourceFilter {

  private String filterName;
  private String filterClass;
  private List initParams;
  private PortletFilter filter;
  private List lifecycle;
  
  public PortletFilterWrapper(String filterName, String filterClass, List initParams, List lifecycle) {
    this.filterName = filterName;
    this.filterClass = filterClass;
    this.initParams = initParams;
    this.lifecycle = lifecycle;
    filter = null;
  }

  public String getFilterName() {
    return filterName;
  }

  public List getInitParam() {
    return initParams;
  }

  public void init(FilterConfig config) throws PortletException {
    if (filter == null) {
      try {
        filter = (PortletFilter) Thread.currentThread().getContextClassLoader().loadClass(filterClass).newInstance();
      } catch (Exception e) {
        throw new PortletException(e);
      }
      filter.init(config);
    }
  }

  public void doFilter(ActionRequest request, ActionResponse response, FilterChain chain) throws IOException, PortletException {
    if (isLifecycleSupported(request))
      ((ActionFilter) filter).doFilter(request, response, chain);
    else
    	chain.doFilter(request, response);
  }
  
  public void doFilter(EventRequest request, EventResponse response, FilterChain chain) throws IOException, PortletException {
    if (isLifecycleSupported(request))
      ((EventFilter) filter).doFilter(request, response, chain);
    else
    	chain.doFilter(request, response);
  }
  
  public void doFilter(RenderRequest request, RenderResponse response, FilterChain chain) throws IOException, PortletException {
    if (isLifecycleSupported(request))
      ((RenderFilter) filter).doFilter(request, response, chain);
    else
    	chain.doFilter(request, response);
  }
  
  public void doFilter(ResourceRequest request, ResourceResponse response, FilterChain chain) throws IOException, PortletException {
    if (isLifecycleSupported(request))
      ((ResourceFilter) filter).doFilter(request, response, chain);
    else
    	chain.doFilter(request, response);
  }

  private boolean isLifecycleSupported(PortletRequest request) {
    Iterator cycles = lifecycle.iterator();
    while (cycles.hasNext()) {
      int cycle = ((Integer) cycles.next()).intValue();
      if ((cycle == Filter.ALL) ||
        ((request instanceof ActionRequest) && (cycle == Filter.ACTION)) ||
        ((request instanceof EventRequest) && (cycle == Filter.EVENT)) ||
        ((request instanceof RenderRequest) && (cycle == Filter.RENDER)) ||
        ((request instanceof ResourceRequest) && (cycle == Filter.RESOURCE))
      ) return true;
    }
    return false;    
  }
  
  public void destroy() {
    filter.destroy();
  }

}
