/**
 * Copyright 2001-2003 The eXo platform SARL All rights reserved.
 * Please look at license.txt in info directory for more license detail.  
 * Created on 12 janv. 2004
 */
package org.exoplatform.services.portletcontainer.plugins.pc.portletAPIImp;

import java.io.IOException;
import java.io.Writer;
import java.net.URLEncoder;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.ArrayList;

import javax.portlet.BaseURL;
import javax.portlet.PortletSecurityException;
import org.exoplatform.Constants;


/**
 * @author  Mestrallet Benjamin
 *          benjmestrallet@users.sourceforge.net
 */
public abstract class BaseURLImp implements BaseURL {
    
  private String markup;
  protected Map parameters = new HashMap();
  protected Map properties = new HashMap ();
  protected boolean isSecure;
  protected boolean setSecureCalled;
  protected String type;
  protected boolean isCurrentlySecured;  
  private boolean defaultEscapeXml;

  public BaseURLImp(String type, String markup, 
                        boolean isCurrentlySecured, boolean defaultEscapeXml) {
    this.type = type;
    this.markup = markup;    
    this.isCurrentlySecured = isCurrentlySecured;
    this.defaultEscapeXml = defaultEscapeXml;
  }

  public void setParameter(String s, String s1) {
    if(s == null)
      throw new IllegalArgumentException("the key given is null");
    if(s1 == null)
      throw new IllegalArgumentException("the value given is null");        
    parameters.put(s, s1);
  }

  public void setParameter(String s, String[] strings) {
    if(s == null)  throw new IllegalArgumentException("the key given is null");
    if(strings == null) throw new IllegalArgumentException("the value given is null");    
    parameters.put(s, strings);
  }

  public void setParameters(Map map) {
    if(map == null)
      throw new IllegalArgumentException("the map given is null");
    if(map.containsKey(null))
      throw new IllegalArgumentException("the map given contains a null key");
    Set keys = map.keySet();
    for (Iterator iter = keys.iterator(); iter.hasNext();) {
      if(! (iter.next() instanceof String ))
        throw new IllegalArgumentException("the map contains a non String key");
    }
    Collection values = map.values();
    for (Iterator iter = values.iterator(); iter.hasNext();) {
      if(! (iter.next() instanceof String[] ))
        throw new IllegalArgumentException("the map contains a non String[] value");
    }           
    parameters = map;
  }

  public Object getParameter(String key) {
    return parameters.get(key);
  }

  public Map getParameterMap() {
    return parameters;
  }
  
  // Spec draft nr.20 new methods  
  
  public void addProperty(String key, String value) {
    if(key == null)  throw new IllegalArgumentException("the property key given is null");
    
    ArrayList <String> propvalues = ((ArrayList)properties.get(key) != null) ? (ArrayList)properties.get(key) : new ArrayList <String> ();
    propvalues.add(value);
    properties.put(key, propvalues);
    
  }

  public void setProperty(String key, String value) {
    if(key == null)  throw new IllegalArgumentException("the property key given is null");
    
    ArrayList <String> propvalues = new ArrayList <String> ();
    propvalues.add(value);
    properties.put(key, propvalues);
    
  }

  public String getPropertyString()
  {
    StringBuffer sb = new StringBuffer();
    
    try {
      Set names = properties.keySet();
      for (Iterator iterator = names.iterator(); iterator.hasNext();) {
        String name = (String) iterator.next() ;
        ArrayList propvalues =  (ArrayList) properties.get(name) ;
        for (int i=0; i<=propvalues.size();i++)
         {
          sb.append(Constants.AMPERSAND);
          sb.append(Constants.PROPERTY_ENCODER);
          sb.append(_encode(name, defaultEscapeXml));
          sb.append("=");
          sb.append(_encode((String)propvalues.get(i), defaultEscapeXml));
         }
      }   
    } catch (Exception e) {
      e.printStackTrace();
    }
    
    return sb.toString();
  }
  
  public void setSecure(boolean b)
          throws PortletSecurityException {
    isSecure = b;
    setSecureCalled = true;
  }

  public abstract String toString(boolean escapeXML);
  
  public String toString() {
    return toString(defaultEscapeXml);
  }

  protected String _encode(String s, boolean escapeXML) {
    if (escapeXML)
      s = encodeChars(s);
    try {
      return URLEncoder.encode(s, "utf-8");
    } catch (java.io.UnsupportedEncodingException e) {
      return s;
    }
  }
  
  protected String encodeChars(String s) {
    return s.
    replace("&", "&amp;").
    replace("<", "&lt;").
    replace(">", "&gt;").
    replace("\"", "&#034;").
    replace("'", "&#039;");
  }
  
  public void write(Writer out) throws IOException {
    out.write(toString(true));
  }

  public void write(Writer out, boolean escapeXML) throws IOException {
    out.write(toString(escapeXML));
  }
  
  
}
