/**
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.
 * Please look at license.txt in info directory for more license detail.
 **/

/**
 * Created by The eXo Platform SARL
 * Author : Mestrallet Benjamin
 *          benjmestrallet@users.sourceforge.net
 * Date: Jul 28, 2003
 * Time: 10:18:25 PM
 */
package org.exoplatform.services.portletcontainer.plugins.pc.portletAPIImp;


import java.io.IOException;
import java.io.PrintWriter;
import java.util.List;
import java.util.Map;
import java.util.Enumeration;
import java.util.Collection;
import java.util.Set;
import java.util.Iterator;

import javax.portlet.*;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletResponseWrapper;
import javax.xml.namespace.QName;

import org.exoplatform.container.ExoContainer;
import org.exoplatform.services.portletcontainer.PortletContainerConf;
import org.exoplatform.services.portletcontainer.pci.Output;
import org.exoplatform.services.portletcontainer.pci.Input;
import org.exoplatform.services.portletcontainer.pci.ActionOutput;
import org.exoplatform.services.portletcontainer.pci.EventOutput;
import org.exoplatform.services.portletcontainer.pci.model.Portlet;
import org.exoplatform.services.portletcontainer.pci.model.Supports;
import org.exoplatform.services.portletcontainer.pci.PortletURLFactory;
import org.exoplatform.services.portletcontainer.plugins.pc.portletAPIImp.PortletURLImp;
import org.exoplatform.services.portletcontainer.plugins.pc.portletAPIImp.ResourceURLImp;
import org.exoplatform.services.portletcontainer.plugins.pc.portletAPIImp.ResponseContext;


public class PortletResponseImp extends HttpServletResponseWrapper
    implements PortletResponse {

  protected ResponseContext resCtx;
  protected ExoContainer cont_;
  protected List customWindowStates_;//TODO
  protected Input input_;
  protected Output output_;
  protected boolean redirectionPossible_;
  protected boolean sendRedirectAlreadyOccured_;
  private Enumeration supportedWindowState_;
  protected Portlet portletDatas_;
  private boolean isCurrentlySecured_;
  private String windowId_;
  
  public PortletResponseImp(ResponseContext resCtx) {
    super(resCtx.getHttpServletResponse());
    this.cont_ = resCtx.getCont();
    this.input_ = resCtx.getInput();
    this.output_ = resCtx.getOutput();
    this.customWindowStates_ = resCtx.getCustomWindowState();
    this.supportedWindowState_ = resCtx.getPortalContext().getSupportedWindowStates();
    this.redirectionPossible_ = true;
    this.sendRedirectAlreadyOccured_ = false;
    this.portletDatas_ = resCtx.getPortletDatas();
    this.windowId_ = resCtx.getWindowID();
    this.isCurrentlySecured_ = resCtx.isSecure();
  }

  public void addProperty(String s, String s1) {
    output_.addProperty(s, s1);
  }

  public void setProperty(String s, String s1) {
    output_.addProperty(s, s1);
  }

  protected String getProperty(String s) {
    return (String) output_.getProperties().get(s);
  }

  protected void removeProperty(String s) {
    output_.getProperties().remove(s);
  }

  public String getNamespace() {
    return (windowId_.replace('-', 'I') + "I").replace('/', 'I');
  }

  public PortletURL createRenderURL() {
  	if(input_.getPortletURLFactory() != null) {
  		return input_.getPortletURLFactory().createPortletURL(PortletURLFactory.RENDER);
  	}
  	return new PortletURLImp(PortletURLFactory.RENDER, 
                             input_.getMarkup(),
														 portletDatas_.getSupports(),
														 isCurrentlySecured_,
//                             customWindowStates_, supportedWindowState_,
                             input_.getBaseURL(),
														 input_.getWindowID(),
                             input_.getEscapeXml());
  }

  public PortletURL createActionURL() {
  	if(input_.getPortletURLFactory() != null) {
  		return input_.getPortletURLFactory().createPortletURL(PortletURLFactory.ACTION);
  	}
    
  	return new PortletURLImp(PortletURLFactory.ACTION,
                             input_.getMarkup(),
														 portletDatas_.getSupports(),
                             isCurrentlySecured_,
//                             customWindowStates_, supportedWindowState_,
                             input_.getBaseURL(),
                             input_.getWindowID(),
                             input_.getEscapeXml());
  }

  public ResourceURL createResourceURL() {
  	if(input_.getPortletURLFactory() != null) {
  		return input_.getPortletURLFactory().createResourceURL(PortletURLFactory.RESOURCE);
  	}
  	return new ResourceURLImp(PortletURLFactory.RESOURCE, input_.getMarkup(),
														 isCurrentlySecured_, 
                             input_.getBaseURL(), input_.getWindowID(), input_.getEscapeXml());
  }
  
  public String encodeURL(String path) {
    if (!path.startsWith("/") && !path.startsWith("http://")) {
      throw new IllegalArgumentException("Path must be started with / or http://") ; 
    }
    // made for TCK com.sun.ts.tests.portlet.api.javax_portlet.PortletRequestDispatcher.ResponseMethodsMiscMiscTestPortlet on weblogic 9.2
    // was: return path;
    if (super.encodeURL(path) == null) {
      return path;
    }    
    return super.encodeURL(path);
  }
  
  public void setRenderParameters(Map map) {
    if(map == null)
      throw new IllegalArgumentException("the map given is null");
    if(map.containsKey(null))
      throw new IllegalArgumentException("the map given contains a null key");
    Set keys = map.keySet();
    for (Iterator iter = keys.iterator(); iter.hasNext();) {
      if(! (iter.next() instanceof String ))
        throw new IllegalArgumentException("the map contains a non String key");
    }
    Collection values = map.values();
    for (Iterator iter = values.iterator(); iter.hasNext();) {
      if(! (iter.next() instanceof String[] ))
        throw new IllegalArgumentException("the map contains a non String[] value");
    }        
    if(sendRedirectAlreadyOccured_)
      throw new IllegalStateException("sendRedirect was already called");    
    redirectionPossible_ = false;
    ((ActionOutput) this.getOutput()).setRenderParameters(map);
  }

  public void setRenderParameter(String s, String s1) {
    if(s == null) throw new IllegalArgumentException("the key given is null");
    if(s1 == null) throw new IllegalArgumentException("the value given is null");    
    if(sendRedirectAlreadyOccured_)
      throw new IllegalStateException("sendRedirect was already called");    
    redirectionPossible_ = false;
    ((ActionOutput) this.getOutput()).setRenderParameter(s, s1);
  }

  public void setRenderParameter(String s, String[] strings) {
    if(s == null)
      throw new IllegalArgumentException("the key given is null");
    if(strings == null)
      throw new IllegalArgumentException("the value given is null");    
    if(sendRedirectAlreadyOccured_)
      throw new IllegalStateException("sendRedirect was already called");    
    redirectionPossible_ = false;
    ((ActionOutput) this.getOutput()).setRenderParameters(s, strings);
  }
  
  //adds for jsr-286
  public Output getOutput() {
    return output_;
  }
  
  public void setOutput(Output o) {
    this.output_ = o;
  }
  
  public boolean isSendRedirectAlreadyOccured() {
    return sendRedirectAlreadyOccured_;
  }
  
  public void setProperty(String key, String value, int scope) {
    // TODO Auto-generated method stub
  }
  
  public void addProperty(Cookie cookie) {
    super.addCookie(cookie);
  }
  
  public void addProperty(String key, org.w3c.dom.Element element) {
    // TODO does it work correctly??
    addProperty(new Cookie(key, element.toString()));
  }

}
