/**
 * Copyright 2001-2003 The eXo platform SARL All rights reserved.
 * Please look at license.txt in info directory for more license detail.  
 * Created on 12 janv. 2004
 */
package org.exoplatform.services.portletcontainer.plugins.pc.portletAPIImp;

import java.io.IOException;
import java.io.Writer;
import java.net.URLEncoder;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.portlet.PortletMode;
import javax.portlet.PortletModeException;
import javax.portlet.PortletURL;
import javax.portlet.WindowState;
import javax.portlet.WindowStateException;

import org.exoplatform.Constants;
import org.exoplatform.services.portletcontainer.pci.Input;
import org.exoplatform.services.portletcontainer.pci.WindowID;
import org.exoplatform.services.portletcontainer.pci.model.CustomWindowState;
import org.exoplatform.services.portletcontainer.pci.model.Supports;


/**
 * @author  Mestrallet Benjamin
 *          benjmestrallet@users.sourceforge.net
 */
public class PortletURLImp extends BaseURLImp implements PortletURL {
    
  private WindowID windowID;
  private String baseURL;
  
  protected List supports;
//  protected Enumeration supportedWindowState;
//  protected List customWindowStates;
  protected WindowState requiredWindowState;
  protected PortletMode requiredPortletMode;
  protected String markup;

  
  public PortletURLImp( String type, 
                        String markup, 
                        List supports, 
                        boolean isCurrentlySecured, 
//                        List customWindowStates,
//                        Enumeration supportedWindowState,
                        String baseURL, 
                        WindowID windowID,
                        boolean defaultEscapeXml) {
    
    super(type, markup, isCurrentlySecured, defaultEscapeXml);
    this.markup = markup;
    this.supports = supports;
//    this.customWindowStates = customWindowStates;
//    this.supportedWindowState = supportedWindowState;
    this.baseURL = baseURL;
    this.windowID = windowID;
  }
  
  public void setWindowState(WindowState windowState)
          throws WindowStateException {
    if(windowState == null){
      throw new WindowStateException("The portlet state is null", windowState); 
    }            
    if (windowState == WindowState.NORMAL || windowState == WindowState.MINIMIZED ||
        windowState == WindowState.MAXIMIZED) {
      requiredWindowState = windowState;
      return;
    }            
        
    boolean supported = false;
    for (Iterator iterator = supports.iterator(); iterator.hasNext();) {
      Supports sp = (Supports) iterator.next();
      if (markup.equals(sp.getMimeType())) {
        List stateList = sp.getWindowState();
        for (Iterator iterator1 = stateList.iterator(); iterator1.hasNext();) {
          String stateString = (String)iterator1.next();
          if (stateString != null && stateString.equalsIgnoreCase(windowState.toString())) {
            supported = true;
            break;
          }
        }
        break;
      }
    }
    if (!supported)
      throw new WindowStateException("The window state " + windowState.toString() + " is not supported by that portlet",
          windowState);

    requiredWindowState = windowState;
  }
    
  public void setPortletMode(PortletMode portletMode) throws PortletModeException {

    if(portletMode == null)
      throw new PortletModeException("The portlet mode is null", portletMode);

    if (portletMode == PortletMode.VIEW) {
      requiredPortletMode = portletMode;
      return;
    }
    
    boolean supported = false;
    for (Iterator iterator = supports.iterator(); iterator.hasNext();) {
      Supports sp = (Supports) iterator.next();
      if (markup.equals(sp.getMimeType())) {
        List modeList = sp.getPortletMode();
        for (Iterator iterator1 = modeList.iterator(); iterator1.hasNext();) {
          String modeString = (String)iterator1.next();
          if (modeString != null && modeString.equalsIgnoreCase(portletMode.toString())) {
            supported = true;
            break;
          }
        }
        break;
      }
    }
    if (!supported)
      throw new PortletModeException("The mode " + portletMode.toString() + " is not supported by that portlet",
              portletMode);

    requiredPortletMode = portletMode;
  }
  
  public PortletMode getPortletMode() {
    return requiredPortletMode;
  }
  
  public WindowState getWindowState() {
    return requiredWindowState;
  }

  public String toString(boolean escapeXML) {
    if(!setSecureCalled && isCurrentlySecured)
      isSecure = true;

    StringBuffer sB = new StringBuffer();
    sB.append(baseURL);
    //sB.append(Constants.AMPERSAND);
    //sB.append(Constants.COMPONENT_PARAMETER);
    //sB.append("=");
    //sB.append(windowID.generateKey());
    sB.append(Constants.AMPERSAND);
    sB.append(Constants.TYPE_PARAMETER);
    sB.append("=");
    sB.append(type);
    sB.append(Constants.AMPERSAND);
    sB.append(Constants.SECURE_PARAMETER);
    sB.append("=");
    sB.append(isSecure);

    if (requiredPortletMode != null) {
      sB.append(Constants.AMPERSAND);
      sB.append(Constants.PORTLET_MODE_PARAMETER);
      sB.append("=");
      sB.append(requiredPortletMode);
    }
    if (requiredWindowState != null) {
      sB.append(Constants.AMPERSAND);
      sB.append(Constants.WINDOW_STATE_PARAMETER);
      sB.append("=");
      sB.append(requiredWindowState);
    }

    Set names = parameters.keySet();
    for (Iterator iterator = names.iterator(); iterator.hasNext();) {
      String name = (String) iterator.next() ;
      Object obj =  parameters.get(name) ;
      if (obj instanceof String) {
        String value = (String) obj ;
        sB.append(Constants.AMPERSAND);
        sB.append(_encode(name, escapeXML));
        sB.append("=");
        sB.append(_encode(value, escapeXML));
      } else {
        String[] values = (String[]) obj ;
        for (int i=0; i < values.length ; i++) {
          sB.append(Constants.AMPERSAND);
          sB.append(_encode(name, escapeXML));
          sB.append("=");
          sB.append(_encode(values[i], escapeXML));
        }
      }
    }
    String propertyString = getPropertyString();
    if (propertyString != "" && propertyString != null)
      {
      //sB.append(Constants.AMPERSAND);
      sB.append(propertyString);
      }
    return sB.toString();
  }
  
}
